@@ -3,13 +3,14 @@
 #' Take time series dataset and fields, then refill the missing date records and other fields.
 #' @param inPath A path which is the location of uncompleted dataset which must be xlsx file
 #' @param sheet A worksheet name of the dataset
-#' @param dateCol Date column
+#' @param dateCol.index Date column
 #' @param outPath A path where the location of xlsx file of completed dataset should be
-#' @param fixedVec  A row of column number which should be kept same values with the original
-#' @param pChanged The column number which should be changed to different value into new record.
-#' @param pChangedNum The value of a specific column which should be put into the new record.
-#' @import openxlsx zoo Hmisc
+#' @param fixedCol.index  A row of column number which should be kept same values with the original
+#' @param uninterpolatedCol.index The column number which should be changed to different value into new record.
+#' @param uninterpolatedCol.newValue The value of a specific column which should be put into the new record.
+#' @import openxlsx zoo Hmisc magrittr
 #' @importFrom utils head install.packages
+#' @importFrom magrittr %>%
 #' @details Real time series sales dataset could be not continuous in 'date' field. e.g., monthly sales data is continuous,
 #'  but discrete in daily data.
 #'
@@ -18,8 +19,8 @@
 #' @author Will Kuan
 #' @examples # Please refer to the examples of function dateRefill.fromData
 #' @export
-dateRefill.fromsFile <-
-  function(inPath, sheet, dateCol, outPath, fixedVec, pChanged, pChangedNum)
+dateRefill.fromFileToExcel <-
+  function(inPath, sheet, dateCol.index, outPath, fixedCol.index, uninterpolatedCol.index, uninterpolatedCol.newValue)
   {
     if(!requireNamespace("openxlsx",quietly = TRUE)){
       install.packages("openxlsx"); requireNamespace("openxlsx", quietly = TRUE)
@@ -42,72 +43,71 @@ dateRefill.fromsFile <-
       requireNamespace("Hmisc", quietly = TRUE)
     }
 
+    if(!requireNamespace("magrittr",quietly = TRUE)){
+      install.packages("magrittr"); requireNamespace("magrittr", quietly = TRUE)
+      #stop("Please install package 'magrittr'. ")
+    }else{
+      requireNamespace("magrittr", quietly = TRUE)
+    }
+
     data <- openxlsx::read.xlsx(inPath, sheet)
 
-    data[,dateCol] <- zoo::as.Date(data[,dateCol], origin = "1899-12-30")
+    #=============
+    data[,dateCol.index] <- zoo::as.Date(data[,dateCol.index], origin = "1899-12-30")
     colNameVector <- colnames(data)
 
-    colnames(data)[dateCol] <- "Date"
+    colnames(data)[dateCol.index] <- "Date"
 
     data$Date <- as.POSIXlt(data$Date) # transform to POSIXlt type
 
     year.list <- levels(factor(data$Date$year + 1900))
 
     ### sorting data
-    inc.order <- order(data$Date, decreasing = FALSE)
-    data <- data[inc.order,]
+    data <- data[order(data$Date, decreasing = FALSE),]
 
     ### building an empty data frame
     final.data <- data.frame(data[,1:length(colNameVector)])
     final.data[,] <- NA
     final.data$Date <- as.POSIXlt(final.data$Date)
 
-    year <- substr(data[1,2],1,4)
+    year <- substr(data[1, dateCol.index],1,4)
     origin <- paste(year, "-01-01", sep = "")
     origin <- zoo::as.Date(origin)
-    diff <- zoo::as.Date(data[1,2])-origin
-    rm(year)
+    diff <- zoo::as.Date(data[1, dateCol.index])-origin
 
-    daySum <- 0
-    for(x in year.list)
-    {
-      daySum <- daySum + Hmisc::yearDays(zoo::as.Date(paste(x, "-01-01", sep = "")))
-    }
-    daySum <- as.numeric(daySum - diff)
+    #=============
+    daySum <- sprintf("%s-01-01", year.list) %>%
+      zoo::as.Date()
+
+    daySum <- mapply(Hmisc::yearDays, daySum) %>%
+      sum()
+
+    daySum <- magrittr::subtract(daySum, diff) %>%
+      as.numeric()
 
     final.data[1:daySum,] <- NA # remove first few null days because data is not start on 1/1
-    final.data[,2] <- seq(data[1,2], by = "1 days", length.out = daySum)
+    final.data[, dateCol.index] <- seq(data[1, dateCol.index], by = "1 days", length.out = daySum)
 
-    #### setting rownames
-    rownames(data) <- c(1:nrow(data))
+    ### duplicate identical column names
+    colnames(data) <- names(final.data)
 
+    ### copy identical record from original data
     my.index <- match(data$Date, as.POSIXlt(final.data$Date))
     final.data[my.index,] <- data[,]
 
-    for(i in daySum:1)
-    {
-      if(!is.na(final.data[i,1])){
-        tag <- i
-        rm(i)
-        break;
-      }else{
-        next;
-      }
-    }
+    ### copy fixedCol.value from original data to new records
+    final.data <- head(final.data, which(final.data$Date == data$Date[length(data$Date)]))
 
-    final.data <- head(final.data,tag)  # cutting off last empty records
-
-    for(j in 1:nrow(final.data)){
-      if(is.na(final.data[j,1])){
-        final.data[j,fixedVec] <- data[1,fixedVec]
-        final.data[j,pChanged] <- pChangedNum
-      }
-    }
+    #==============
+    final.data[which(is.na(final.data[, fixedCol.index[1]])), fixedCol.index] <- data[1, fixedCol.index]
+    final.data[which(is.na(final.data[, uninterpolatedCol.index[1]])), uninterpolatedCol.index] <- uninterpolatedCol.newValue
 
+    #==============
     final.data$Date <- zoo::as.Date(final.data$Date) # for correcting date time in excel
 
     colnames(final.data) <- colNameVector
 
+
     ### output data into excel file
     openxlsx::write.xlsx(final.data, file = outPath)
 
@@ -117,12 +117,13 @@ dateRefill.fromsFile <-
 #'
 #' Take time series dataset and fields, then refill the missing date records and other fields.
 #' @param data The data.frame dataset which is ready to be processed
-#' @param dateCol Date column
-#' @param fixedVec  A row of column number which should be kept same values with the original
-#' @param pChanged The column number which should be changed to different value into new record.
-#' @param pChangedNum The value of a specific column which should be put into the new record.
-#' @import zoo Hmisc
+#' @param dateCol.index Date column
+#' @param fixedCol.index  A row of column number which should be kept same values with the original
+#' @param uninterpolatedCol.index The column number which should be changed to different value into new record.
+#' @param uninterpolatedCol.newValue The value of a specific column which should be put into the new record.
+#' @import zoo Hmisc magrittr
 #' @importFrom utils head install.packages
+#' @importFrom magrittr %>%
 #' @return The dataset which is completed.
 #' @details Real time series sales dataset could be not continuous in 'date' field. e.g., monthly sales data is continuous,
 #'  but discrete in daily data.
@@ -132,10 +133,10 @@ dateRefill.fromsFile <-
 #' @author Will Kuan
 #' @examples # mydata <- data.example
 #' # mydata.final <- dateRefill.fromData(data = mydata,dateCol = 2,fixedVec = c(3:10),
-#' #                                     pChanged = 11,pChangedNum = 0)
+#' #                                     uninterpolatedCol.index = 11,uninterpolatedCol.newValue = 0)
 #' @export
 dateRefill.fromData <-
-  function(data, dateCol, fixedVec, pChanged, pChangedNum)
+  function(data, dateCol.index, fixedCol.index, uninterpolatedCol.index, uninterpolatedCol.newValue)
   {
     if(!requireNamespace("zoo", quietly = TRUE)){
       install.packages("zoo"); requireNamespace("zoo", quietly = TRUE)
@@ -151,68 +152,66 @@ dateRefill.fromData <-
       requireNamespace("Hmisc", quietly = TRUE)
     }
 
+    if(!requireNamespace("magrittr",quietly = TRUE)){
+      install.packages("magrittr"); requireNamespace("magrittr", quietly = TRUE)
+      #stop("Please install package 'magrittr'. ")
+    }else{
+      requireNamespace("magrittr", quietly = TRUE)
+    }
+
     data <- data.frame(data)
 
-    data[,dateCol] <- zoo::as.Date(data[,dateCol], origin = "1899-12-30")
+    #=============
+    data[,dateCol.index] <- zoo::as.Date(data[,dateCol.index], origin = "1899-12-30")
     colNameVector <- colnames(data)
 
-    colnames(data)[dateCol] <- "Date"
+    colnames(data)[dateCol.index] <- "Date"
 
     data$Date <- as.POSIXlt(data$Date) # transform to POSIXlt type
 
     year.list <- levels(factor(data$Date$year + 1900))
 
     ### sorting data
-    inc.order <- order(data$Date, decreasing = FALSE)
-    data <- data[inc.order,]
+    data <- data[order(data$Date, decreasing = FALSE),]
 
     ### building an empty data frame
     final.data <- data.frame(data[,1:length(colNameVector)])
     final.data[,] <- NA
     final.data$Date <- as.POSIXlt(final.data$Date)
 
-    year <- substr(data[1,2],1,4)
+    year <- substr(data[1, dateCol.index],1,4)
     origin <- paste(year, "-01-01", sep = "")
     origin <- zoo::as.Date(origin)
-    diff <- zoo::as.Date(data[1,2])-origin
-    rm(year)
+    diff <- zoo::as.Date(data[1, dateCol.index])-origin
 
-    daySum <- 0
-    for(x in year.list)
-    {
-      daySum <- daySum + Hmisc::yearDays(zoo::as.Date(paste(x, "-01-01", sep = "")))
-    }
-    daySum <- as.numeric(daySum - diff)
+    #=============
+    daySum <- sprintf("%s-01-01", year.list) %>%
+      zoo::as.Date()
+
+    daySum <- mapply(Hmisc::yearDays, daySum) %>%
+      sum()
+
+    daySum <- magrittr::subtract(daySum, diff) %>%
+      as.numeric()
 
     final.data[1:daySum,] <- NA # remove first few null days because data is not start on 1/1
-    final.data[,2] <- seq(data[1,2], by = "1 days", length.out = daySum)
+    final.data[, dateCol.index] <- seq(data[1, dateCol.index], by = "1 days", length.out = daySum)
 
-    #### setting rownames
-    rownames(data) <- c(1:nrow(data))
+    ### duplicate identical column names
+    colnames(data) <- names(final.data)
 
+    ### copy identical record from original data
     my.index <- match(data$Date, as.POSIXlt(final.data$Date))
     final.data[my.index,] <- data[,]
 
-    for(i in daySum:1)
-    {
-      if(!is.na(final.data[i,1])){
-        tag <- i
-        rm(i)
-        break;
-      }else{
-        next;
-      }
-    }
+    ### copy fixedCol.value from original data to new records
+    final.data <- head(final.data, which(final.data$Date == data$Date[length(data$Date)]))
 
-    final.data <- head(final.data,tag)  # cutting off last empty records
-
-    for(j in 1:nrow(final.data)){
-      if(is.na(final.data[j,1])){
-        final.data[j,fixedVec] <- data[1,fixedVec]
-        final.data[j,pChanged] <- pChangedNum
-      }
-    }
+    #==============
+    final.data[which(is.na(final.data[, fixedCol.index[1]])), fixedCol.index] <- data[1, fixedCol.index]
+    final.data[which(is.na(final.data[, uninterpolatedCol.index[1]])), uninterpolatedCol.index] <- uninterpolatedCol.newValue
 
+    #==============
     final.data$Date <- zoo::as.Date(final.data$Date) # for correcting date time in excel
 
     colnames(final.data) <- colNameVector
